@@ -15,10 +15,12 @@
 import warnings
 
 import debtcollector
+from debtcollector.fixtures import disable
 from debtcollector import moves
 from debtcollector import removals
 from debtcollector import renames
 from debtcollector.tests import base as test_base
+from debtcollector import updating
 
 
 @renames.renamed_kwarg('blip', 'blop')
@@ -31,6 +33,16 @@ def blip_blop_2(blip=1, blop=1):
     return (blip, blop)
 
 
+@renames.renamed_kwarg('blip', 'blop', replace=True)
+def blip_blop_3(blop=1):
+    return blop
+
+
+@updating.updated_kwarg_default_value('type', 'cat', 'feline')
+def blip_blop_blip(type='cat'):
+    return "The %s meowed quietly" % type
+
+
 class WoofWoof(object):
     @property
     def bark(self):
@@ -66,6 +78,17 @@ class KittyKat(object):
         return 'supermeow'
 
 
+class Giraffe(object):
+    color = 'orange'
+    colour = moves.moved_read_only_property('colour', 'color')
+
+    @property
+    def height(self):
+        return 2
+
+    heightt = moves.moved_read_only_property('heightt', 'height')
+
+
 class NewHotness(object):
     def hot(self):
         return 'cold'
@@ -91,6 +114,14 @@ def blue_comet():
     return True
 
 
+def yellow_sun():
+    """Yellow."""
+    return True
+
+
+yellowish_sun = moves.moved_function(yellow_sun, 'yellowish_sun', __name__)
+
+
 @removals.remove()
 class EFSF(object):
     pass
@@ -101,11 +132,39 @@ class EFSF_2(object):
     pass
 
 
+@removals.removed_class("StarLord")
+class StarLord(object):
+    def __init__(self):
+        self.name = "star"
+
+
+class StarLordJr(StarLord):
+    def __init__(self, name):
+        super(StarLordJr, self).__init__()
+        self.name = name
+
+
 class ThingB(object):
     @removals.remove()
     def black_tristars(self):
         pass
 
+    @removals.removed_property
+    def green_tristars(self):
+        return 'green'
+
+    @green_tristars.setter
+    def green_tristars(self, value):
+        pass
+
+    @green_tristars.deleter
+    def green_tristars(self):
+        pass
+
+    @removals.removed_property(message="stop using me")
+    def green_blue_tristars(self):
+        return 'green-blue'
+
     @removals.remove(category=PendingDeprecationWarning)
     def blue_tristars(self):
         pass
@@ -191,6 +250,14 @@ class MovedPropertyTest(test_base.TestCase):
         self.assertEqual('woof', dog.burk)
         self.assertEqual('woof', dog.bark)
 
+    def test_readonly_move(self):
+        with warnings.catch_warnings(record=True) as capture:
+            warnings.simplefilter("always")
+            self.assertEqual('orange', Giraffe.colour)
+            g = Giraffe()
+            self.assertEqual(2, g.heightt)
+        self.assertEqual(2, len(capture))
+
     def test_warnings_emitted(self):
         dog = WoofWoof()
         with warnings.catch_warnings(record=True) as capture:
@@ -217,6 +284,34 @@ class MovedPropertyTest(test_base.TestCase):
         self.assertEqual(0, len(capture))
 
 
+class DisabledTest(test_base.TestCase):
+    def test_basics(self):
+        dog = WoofWoof()
+        c = KittyKat()
+        with warnings.catch_warnings(record=True) as capture:
+            warnings.simplefilter("always")
+            with disable.DisableFixture():
+                self.assertTrue(yellowish_sun())
+                self.assertEqual('woof', dog.berk)
+                self.assertEqual('supermeow', c.meow())
+        self.assertEqual(0, len(capture))
+
+
+class MovedFunctionTest(test_base.TestCase):
+    def test_basics(self):
+        self.assertTrue(yellowish_sun())
+        self.assertTrue(yellow_sun())
+        self.assertEqual("Yellow.", yellowish_sun.__doc__)
+
+    def test_warnings_emitted(self):
+        with warnings.catch_warnings(record=True) as capture:
+            warnings.simplefilter("always")
+            self.assertTrue(yellowish_sun())
+        self.assertEqual(1, len(capture))
+        w = capture[0]
+        self.assertEqual(DeprecationWarning, w.category)
+
+
 class MovedMethodTest(test_base.TestCase):
     def test_basics(self):
         c = KittyKat()
@@ -255,6 +350,8 @@ class RenamedKwargTest(test_base.TestCase):
         self.assertEqual((2, 1), blip_blop(blip=2))
         self.assertEqual((1, 2), blip_blop(blop=2))
         self.assertEqual((2, 2), blip_blop(blip=2, blop=2))
+        self.assertEqual(2, blip_blop_3(blip=2))
+        self.assertEqual(2, blip_blop_3(blop=2))
 
     def test_warnings_emitted(self):
         with warnings.catch_warnings(record=True) as capture:
@@ -263,6 +360,12 @@ class RenamedKwargTest(test_base.TestCase):
         self.assertEqual(1, len(capture))
         w = capture[0]
         self.assertEqual(DeprecationWarning, w.category)
+        with warnings.catch_warnings(record=True) as capture:
+            warnings.simplefilter("always")
+            self.assertEqual(2, blip_blop_3(blip=2))
+        self.assertEqual(1, len(capture))
+        w = capture[0]
+        self.assertEqual(DeprecationWarning, w.category)
 
     def test_warnings_emitted_classmethod(self):
         with warnings.catch_warnings(record=True) as capture:
@@ -289,6 +392,28 @@ class RenamedKwargTest(test_base.TestCase):
             warnings.simplefilter("always")
             self.assertEqual((1, 2), blip_blop(blop=2))
         self.assertEqual(0, len(capture))
+        with warnings.catch_warnings(record=True) as capture:
+            warnings.simplefilter("always")
+            self.assertEqual(2, blip_blop_3(blop=2))
+        self.assertEqual(0, len(capture))
+
+
+class UpdatedArgsTest(test_base.TestCase):
+    def test_basic(self):
+        with warnings.catch_warnings(record=True) as capture:
+            warnings.simplefilter("always")
+            self.assertEqual('The cat meowed quietly', blip_blop_blip())
+        self.assertEqual(1, len(capture))
+        w = capture[0]
+        self.assertEqual(FutureWarning, w.category)
+
+    def test_kwarg_set(self):
+        with warnings.catch_warnings(record=True) as capture:
+            warnings.simplefilter("always")
+            self.assertEqual(
+                'The kitten meowed quietly',
+                blip_blop_blip(type='kitten'))
+        self.assertEqual(0, len(capture))
 
 
 class RemovalTests(test_base.TestCase):
@@ -334,6 +459,27 @@ class RemovalTests(test_base.TestCase):
             self.assertEqual(2, f())
         self.assertEqual(0, len(capture))
 
+    def test_warnings_emitted_property(self):
+        with warnings.catch_warnings(record=True) as capture:
+            warnings.simplefilter("always")
+            o = ThingB()
+            self.assertEqual('green', o.green_tristars)
+            o.green_tristars = 'b'
+            del o.green_tristars
+        self.assertEqual(3, len(capture))
+        w = capture[0]
+        self.assertEqual(DeprecationWarning, w.category)
+
+    def test_warnings_emitted_property_custom_message(self):
+        with warnings.catch_warnings(record=True) as capture:
+            warnings.simplefilter("always")
+            o = ThingB()
+            self.assertEqual('green-blue', o.green_blue_tristars)
+        self.assertEqual(1, len(capture))
+        w = capture[0]
+        self.assertIn('stop using me', str(w.message))
+        self.assertEqual(DeprecationWarning, w.category)
+
     def test_warnings_emitted_function_args(self):
         with warnings.catch_warnings(record=True) as capture:
             warnings.simplefilter("always")
@@ -382,6 +528,24 @@ class RemovalTests(test_base.TestCase):
         w = capture[0]
         self.assertEqual(PendingDeprecationWarning, w.category)
 
+    def test_pending_warnings_emitted_class_direct(self):
+        with warnings.catch_warnings(record=True) as capture:
+            warnings.simplefilter("always")
+            s = StarLord()
+        self.assertEqual(1, len(capture))
+        w = capture[0]
+        self.assertEqual(DeprecationWarning, w.category)
+        self.assertEqual("star", s.name)
+
+    def test_pending_warnings_emitted_class_inherit(self):
+        with warnings.catch_warnings(record=True) as capture:
+            warnings.simplefilter("always")
+            s = StarLordJr("star_jr")
+        self.assertEqual(1, len(capture))
+        w = capture[0]
+        self.assertEqual(DeprecationWarning, w.category)
+        self.assertEqual("star_jr", s.name)
+
     def test_warnings_emitted_instancemethod(self):
         zeon = ThingB()
         with warnings.catch_warnings(record=True) as capture:
